@@ -1,4 +1,4 @@
-;; Copyright (C) 2015-2016 Free Software Foundation, Inc
+;; Copyright (C) 2015-2016, 2019 Free Software Foundation, Inc
 
 ;; Author: Rocky Bernstein <rocky@gnu.org>
 
@@ -17,6 +17,8 @@
 ;;; Emacs Commands to associate or attach a source buffer to a command
 ;;; buffer and vice versa.
 
+(eval-when-compile (require 'cl-lib))   ;For cl-remove-if-not.
+
 (require 'load-relative)
 (require-relative-list  '("buffer/command" "buffer/source")
 			"realgud-buffer-")
@@ -27,6 +29,10 @@
 (declare-function realgud-srcbuf-init-or-update       'realgud-source)
 (declare-function realgud-short-key-mode-setup        'realgud-shortkey)
 
+(defvar realgud:attach-cmdbuf-history nil "attach command buffer history list'.")
+
+
+;;;###autoload
 (defun realgud:attach-source-buffer(srcbuf)
   "Associate a source buffer with the current command buffer"
   (interactive "bsource buffer: ")
@@ -47,10 +53,15 @@
     )
   )
 
-(defun realgud:attach-command-buffer(cmdbuf)
+;;;###autoload
+(defun realgud:attach-cmd-buffer(cmdbuf)
   "Associate a command buffer with the current source buffer"
 
-  (interactive "bcommand buffer: ")
+  (interactive
+   (list
+    (completing-read "Choose a realgud command buffer: "
+		     (realgud:attach-list-command-buffers) nil t nil
+		     'realgud:attach-cmdbuf-history nil)))
   (if (stringp cmdbuf) (setq cmdbuf (get-buffer cmdbuf)))
   (let* ((srcbuf (current-buffer))
 	 (shortkey-mode?))
@@ -60,9 +71,14 @@
       (unless (get-buffer-process (current-buffer))
 	(warn "Can't find a process for command buffer %s" (current-buffer)))
       (setq shortkey-mode? (realgud-sget 'cmdbuf-info 'src-shortkey?)))
+    (add-to-list 'realgud:attach-cmdbuf-history (buffer-name cmdbuf))
     (realgud-cmdbuf-add-srcbuf srcbuf)
     (realgud-srcbuf-init-or-update srcbuf cmdbuf)
     (if shortkey-mode? (realgud-short-key-mode-setup 't)))
   )
 
+(defun realgud:attach-list-command-buffers()
+  (mapcar 'buffer-name (cl-remove-if-not 'realgud-cmdbuf? (buffer-list))))
+
+
 (provide-me "realgud-")
