@@ -1,4 +1,4 @@
-;; Copyright (C) 2014, 2016 Free Software Foundation, Inc
+;; Copyright (C) 2014, 2016, 2018-2019 Free Software Foundation, Inc
 
 ;; Author: Rocky Bernstein <rocky@gnu.org>
 
@@ -9,7 +9,6 @@
 
 ;; We use gud to handle the classpath-to-filename mapping
 (require 'gud)
-(require 'compile) ;; for compilation-find-file
 
 (require 'load-relative)
 (require-relative-list '("../../common/track"
@@ -26,6 +25,7 @@
 (declare-function realgud-query-cmdline      'realgud-core)
 (declare-function realgud-suggest-invocation 'realgud-core)
 (declare-function realgud:file-loc-from-line 'realgud-file)
+(declare-function realgud:find-file          'realgud-file)
 
 ;; FIXME: I think the following could be generalized and moved to
 ;; realgud-... probably via a macro.
@@ -54,54 +54,56 @@ ca.mgcill.rocky.snpEff.main => ca/mcgill/rocky/snpEff"
       (setq str (replace-regexp-in-string "\\." "/" str))
       str)
 
-(defvar realgud:jdb-file-remap (make-hash-table :test 'equal)
-  "How to remap Java files in jdb when we otherwise can't find in
-  the filesystem. The hash key is the file string we saw, and the
-  value is associated filesystem string presumably in the
-  filesystem")
-
-(defun realgud:jdb-find-file(filename)
+(defun realgud:jdb-find-file(marker filename directory)
   "A find-file specific for java/jdb. We use `gdb-jdb-find-source' to map a
 name to a filename. Failing that we can add on .java to the name. Failing that
-we will prompt for a mapping and save that in `realgud:jdb-file-remap' when
-that works."
+we will prompt for a mapping and save that the remap."
   (let* ((transformed-file)
+	 (cmdbuf (realgud-get-cmdbuf))
+	 (ignore-re-file-list (realgud-cmdbuf-ignore-re-file-list cmdbuf))
+	 (filename-remap-alist (realgud-cmdbuf-filename-remap-alist))
 	 (stripped-filename (realgud:strip filename))
 	 (gud-jdb-filename (gud-jdb-find-source stripped-filename))
+	 (remapped-filename
+	  (assoc filename filename-remap-alist))
 	)
     (cond
      ((and gud-jdb-filename (file-exists-p gud-jdb-filename))
       gud-jdb-filename)
      ((file-exists-p (setq transformed-file (concat stripped-filename ".java")))
       transformed-file)
-     ('t
-      (if (gethash stripped-filename realgud:jdb-file-remap)
-	  (let ((remapped-filename))
-	    (setq remapped-filename (gethash stripped-filename realgud:jdb-file-remap))
-	    (if (file-exists-p remapped-filename)
-		remapped-filename
-	      ;; else
-	      (and (remhash filename realgud-file-remap) nil)))
+     ((realgud:file-ignore filename ignore-re-file-list)
+      (message "tracking ignored for %s" filename) nil)
+     (t
+      (if remapped-filename
+	  (if (file-exists-p (cdr remapped-filename))
+	      (cdr remapped-filename)
+     	    ;; else remove from map since no find
+     	    (and (realgud-cmdbuf-filename-remap-alist=
+     		  (delq (assoc remapped-filename filename-remap-alist)
+     			filename-remap-alist))
+     		  nil))
 	;; else
 	(let ((remapped-filename)
 	      (guess-filename (realgud:jdb-dot-to-slash filename)))
 	  (setq remapped-filename
 		(buffer-file-name
-		 (compilation-find-file (point-marker) guess-filename
-					nil "%s.java")))
+		 (realgud:find-file marker guess-filename
+				    directory "%s.java")))
 	  (when (and remapped-filename (file-exists-p remapped-filename))
-	    (puthash stripped-filename remapped-filename realgud:jdb-file-remap)
-	    remapped-filename
+     	    (realgud-cmdbuf-filename-remap-alist=
+     	     (cons
+     	      (cons filename remapped-filename)
+     	      filename-remap-alist))
 	    ))
 	))
-     ))
-  )
+     )))
 
 (defun realgud:jdb-loc-fn-callback(text filename lineno source-str
-					ignore-file-re cmd-mark)
+					cmd-mark directory)
   (realgud:file-loc-from-line filename lineno
 			      cmd-mark source-str nil
-			      ignore-file-re 'realgud:jdb-find-file))
+			      'realgud:jdb-find-file directory))
 
 (defun realgud:jdb-parse-cmd-args (orig-args)
   "Parse command line ARGS for the annotate level and name of script to debug.
@@ -112,7 +114,7 @@ We return the a list containing
 
 * the command debugger (e.g. jdb)
 
-* debugger command rguments if any - a list of strings
+* debugger command arguments if any - a list of strings
 
 * the script name and its arguments - list of strings
 
